using System;
using System.Collections.Generic;
using System.Text;

namespace IronPython.Console
{
    internal class CommandHistory
    {
        #region Privates

        private int _max;
        private int _lastId = 0;
        private readonly SortedList<int, string> _history = new SortedList<int, string>();

        #endregion Privates

        #region Constructor

        public CommandHistory()
            : this (100)
        {
        }

        public CommandHistory(int maxCommands)
        {
            _max = maxCommands;
            Record("");
        }

        #endregion Constructor

        #region Properties

        internal int LastCommandId
        {
            get
            {
                return _history.Keys[_history.Keys.Count - 1];
            }
        }

        internal int FirstCommandId
        {
            get
            {
                return _history.Keys[0];
            }
        }

        internal string this[int commandId]
        {
            get
            {
                return _history[commandId];
            }
        }

        #endregion Properties

        #region Methods

        public void Record(string command)
        {
            /// if this is the first command to be recorded, we get rid of the dummy
            /// empty command we stored.
            if (_history.Count == 1 && string.IsNullOrEmpty(_history[0]))
                _history[0] = command;
            else
            {
                /// The last command that was entered, is the last command in the list
                _history.Add(_lastId++, command);
                if (_history.Count > _max)
                    _history.RemoveAt(0);
            }
        }

        public Iterator GetEnumerator()
        {
            return new Iterator(this);
        }

        #endregion Methods

        #region class Iterator

        /// <summary>
        /// Starts the iterator from the last item in the history - 
        /// the freshest command that had been entered.
        /// older commands are previous.
        /// newer commands are next. 
        /// </summary>
        public class Iterator
        {
            #region Privates

            private readonly CommandHistory _parent;
            private int _id;

            #endregion Privates

            #region Constructor

            internal Iterator(CommandHistory parent)
            {
                _parent = parent;
                _id = _parent.LastCommandId;
            }

            #endregion Constructor

            #region Properties

            public string Current
            {
                get
                {
                    return _parent[_id];
                }
            }

            public bool IsValid
            {
                get
                {
                    return _parent.FirstCommandId <= _id && _id <= _parent.LastCommandId;
                }
            }

            public bool HasNext
            {
                get
                {
                    System.Diagnostics.Debug.Assert (IsValid);
                    return _id < _parent.LastCommandId;
                }
            }

            public bool HasPrevious
            {
                get
                {
                    System.Diagnostics.Debug.Assert(IsValid);
                    return _parent.FirstCommandId < _id;
                }
            }

            #endregion Properties

            #region Methods

            public bool MoveNext()
            {
                if (HasNext)
                {
                    ++_id;
                    return true;
                }
                else
                    return false;
            }

            public bool MovePrevious ()
            {
                if (HasPrevious)
                {
                    --_id;
                    return true;
                }
                else
                    return false;
            }

            public void Reset()
            {
                _id = _parent.LastCommandId;
            }

            #endregion Methods
        }

        #endregion class Iterator
    }
}
