using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Text;
using System.IO;

namespace IronPython.Console
{
    public class Driver
    {
        #region Delegate declarations

        #region EventArgs

        #region CursorMovedEventArgs

        public class CursorMovedEventArgs : System.EventArgs
        {
            public CursorMovedEventArgs(int newOffset)
            {
                NewOffset = newOffset;
            }

            /// <summary>
            /// returns the new offset from the start of the text that cursor should be in
            /// </summary>
            public readonly int NewOffset;
        }

        #endregion CursorMovedEventArgs

        #region TextChangedEventArgs

        public class TextChangedEventArgs : CursorMovedEventArgs
        {
            public TextChangedEventArgs(string newText, int newOffset)
                : base (newOffset)
            {
                NewText = newText;
            }

            /// <summary>
            /// returns the new text of the current command
            /// </summary>
            public readonly string NewText;

            public override string ToString()
            {
                return GetType().FullName + "(\"" + NewText + "\", " + NewText + ")";
            }
        }

        #endregion TextChangedEventArgs

        #region ResponseEventArgs

        public enum ResponseStyle
        {
            Output,
            Error,
            Prompt
        }

        public class ResponseEventArgs : System.EventArgs
        {
            public ResponseEventArgs(string response, ResponseStyle style)
            {
                Response = response;
                Style = style;
            }

            /// <summary>
            /// returns the response that the engine has returned for the last statement
            /// </summary>
            public readonly string Response;

            /// <summary>
            /// returns the prompt that the engine returned
            /// </summary>
            public readonly ResponseStyle Style;

            public override string ToString()
            {
                return GetType ().FullName + "(\"" + Response + "\", " + Style.ToString () + ")"; 
            }

            /// <summary>
            /// Determines whether the specified <see cref="T:System.Object"></see> is equal to the current <see cref="T:System.Object"></see>.
            /// </summary>
            /// <param name="obj">The <see cref="T:System.Object"></see> to compare with the current <see cref="T:System.Object"></see>.</param>
            /// <returns>
            /// true if the specified <see cref="T:System.Object"></see> is equal to the current <see cref="T:System.Object"></see>; otherwise, false.
            /// </returns>
            public override bool Equals(object obj)
            {
                if (obj is ResponseEventArgs)
                {
                    ResponseEventArgs other = (ResponseEventArgs) obj;
                    if (Response.Equals (other.Response))
                        if (Style.Equals (other.Style))
                            return true;

                }

                return false;
            }
        }

        #endregion ResponseEventArgs

        #endregion EventArgs

        #region Delegates

        public delegate void CursorMovedEvent (Driver sender, CursorMovedEventArgs args);
        public delegate void TextChangedEvent (Driver sender, TextChangedEventArgs args);
        public delegate void ResponseEvent (Driver sender, ResponseEventArgs args);

        #endregion Delegates

        #endregion Delegate declarations

        #region Events
        public event CursorMovedEvent CursorMoved;
        public event TextChangedEvent TextChanged;
        public event ResponseEvent Response;
        public event ResponseEvent Prompt;

        protected void OnCursorMoved(int newPosition)
        {
            CursorMovedEvent e = CursorMoved;
            if (e != null)
                e (this, new CursorMovedEventArgs (newPosition));
        }

        protected void OnTextChanged(string newText, int newPosition)
        {
            TextChangedEvent e = TextChanged;
            if (e != null)
                e (this, new TextChangedEventArgs (newText, newPosition));
        }
        protected void OnResponse(string response, ResponseStyle style)
        {
            ResponseEvent e = this.Response;
            if (e != null)
                e (this, new ResponseEventArgs (response, style));
        }
        protected void OnPrompt(string promptString)
        {
            ResponseEvent e = this.Prompt;
            if (e != null)
                e(this, new ResponseEventArgs(promptString, ResponseStyle.Prompt));
        }

        #endregion events

        #region Properties

        /// <summary>
        /// returns the current cursor position in the current line
        /// </summary>
        /// <value>The cursor position.</value>
        public int CurrentEditingPosition
        {
            get
            {
                return _currentCommandIndex;
            }
        }

        /// <summary>
        /// returns the current line that is being edited
        /// </summary>
        /// <value>The line.</value>
        public string CurrentEditingText
        {
            get
            {
                return _currentCommand.ToString ();
            }
        }

        #endregion Properties


        #region CurrentCommand

        private readonly StringBuilder _currentCommand = new StringBuilder();
        private int _currentCommandIndex = 0;

        private bool IsIndentationIndex()
        {
            string leftSide = _currentCommand.ToString ().Substring (0, _currentCommandIndex);
            return (leftSide.Trim ().Length == 0);
        }

        public void BackspacePressed()
        {
            if (_currentCommandIndex > 0)
            {
                if (_options.AutoIndent && IsIndentationIndex())
                {
                    _currentCommandIndex -= _options.AutoIndentSize;
                    _currentCommand.Remove(_currentCommandIndex, _options.AutoIndentSize);
                }
                else
                {
                    --_currentCommandIndex;
                    _currentCommand.Remove(_currentCommandIndex, 1);
                }
                
                OnTextChanged(_currentCommand.ToString (), _currentCommandIndex);
            }
        }

        public void DelPressed()
        {
            if (_currentCommandIndex >= _currentCommand.Length)
                return;

            if (_options.AutoIndent && _currentCommand [_currentCommandIndex] == ' ' && IsIndentationIndex())
                _currentCommand.Remove(_currentCommandIndex, _options.AutoIndentSize);
            else
                _currentCommand.Remove(_currentCommandIndex, 1);

            OnTextChanged(_currentCommand.ToString (), _currentCommandIndex);
        }
        public void LeftPressed()
        {
            if (_currentCommandIndex > 0)
            {
                if (_options.AutoIndent && IsIndentationIndex())
                    _currentCommandIndex -= _options.AutoIndentSize;
                else
                    --_currentCommandIndex;

                OnCursorMoved(_currentCommandIndex);
            }
        }

        public void RightPressed()
        {
            if (_currentCommandIndex < _currentCommand.Length)
            {
                if (_options.AutoIndent && IsIndentationIndex())
                    _currentCommandIndex += _options.AutoIndentSize;
                else
                    ++_currentCommandIndex;
                
                OnCursorMoved(_currentCommandIndex);
            }
        }
        public void CharKeyPressed(char c)
        {
            if (Char.IsControl(c))
                return;

            if (_options.AutoIndent && c == ' '){
                /// if this is the beginning of the line (before any text)
                /// than since we are auto-indenting, we shall insert spaces to conserve the indentation
                if (IsIndentationIndex ()) {
                    string indent = new string (' ', _options.AutoIndentSize);
                    _currentCommand.Insert(_currentCommandIndex, indent);
                    _currentCommandIndex += _options.AutoIndentSize;

                    OnTextChanged(_currentCommand.ToString (), _currentCommandIndex);
                    return;
                }
            }

            _currentCommand.Insert(_currentCommandIndex++, c);
            OnTextChanged(_currentCommand.ToString(), _currentCommandIndex);
        }

        public void Reset()
        {
            _currentCommand.Length = 0;
            _currentCommandIndex = 0;
        }

        #endregion CurrentCommand

        #region CurrentStatement

        private class CurrentStatement
        {
            private StringBuilder _buffer = new StringBuilder ();
            private int _autoIndent = 0;
            private bool _completeStatement = true;
            private Driver _driver;

            private int AutoIndentSize
            {
                get
                {
                    return _driver._options.AutoIndentSize;
                }
            }

            public CurrentStatement(Driver driver)
            {
                _driver = driver;
            }

            public void AddLine(string line)
            {
                bool _allowIncompleteStatement;
                if (_buffer.Length == 0)
                    _allowIncompleteStatement = line.TrimEnd ().EndsWith (":");
                else
                    _allowIncompleteStatement = !TreatAsBlankLine (line);
                
                _buffer.Append(line);
                _buffer.Append("\n");

                if (_driver._engine.ParseInteractiveInput(_buffer.ToString(), _allowIncompleteStatement))
                {
                    _completeStatement = true;
                }
                else
                {
                    _completeStatement = false;
                    _autoIndent = IronPython.Compiler.Parser.GetNextAutoIndentSize(
                        _buffer.ToString(),
                        AutoIndentSize
                        );
                }
            }

            public int AutoIndent
            {
                get
                {
                    return _autoIndent;
                }
            }

            public bool IsCompleteStatement
            {
                get
                {
                    return _completeStatement;
                }
            }

            public override string ToString()
            {
                return _buffer.ToString ();
            }

            public void Reset()
            {
                _buffer.Length = 0;
                _autoIndent = 0;
                _completeStatement = true;
            }
        }

        #endregion CurrentStatement


        private readonly CurrentStatement _currentStatement;
        private readonly ConsoleOptions _options;
        private readonly CommandHistory _history = new CommandHistory ();
        private readonly CommandHistory.Iterator _historyIterator;
        private readonly IronPython.Hosting.PythonEngine _engine;
        private readonly System.IO.MemoryStream _stdOutput;
        private readonly System.IO.MemoryStream _stdError;
        private readonly System.IO.MemoryStream _stdInput;


        public Driver()
            : this(new ConsoleOptions())
        {
        }

        public Driver(ConsoleOptions options)
        {
            _stdInput = new MemoryStream ();
            _stdOutput = new MemoryStream();
            _stdError = new MemoryStream();

            _options = options;
            _currentStatement = new CurrentStatement(this);

            _engine = new IronPython.Hosting.PythonEngine(_options);
            //_engine.SetStandardInput(stdIn);
            _engine.SetStandardOutput(_stdOutput);
            _engine.SetStandardError(_stdError);

            _historyIterator = _history.GetEnumerator();
        }

        public void EnterPressed()
        {
            /// record this line in the history
            _history.Record(_currentCommand.ToString ());
            _historyIterator.Reset();

            try
            {
                _currentStatement.AddLine(_currentCommand.ToString());
            }
            catch (Exception e)
            {
                Reset();
                _currentStatement.Reset();

                OnResponse(_engine.FormatException(e), ResponseStyle.Error);
                OnPrompt(_engine.Sys.ps1.ToString());
                return;
            }

            if (_currentStatement.IsCompleteStatement)
            {
                try
                {
                    _stdOutput.Seek(0, SeekOrigin.Begin);
                    _stdOutput.SetLength(0);

                    _stdError.Seek(0, SeekOrigin.Begin);
                    _stdError.SetLength(0);
                    
                    _engine.ExecuteToConsole(_currentStatement.ToString());
                    _stdOutput.Flush();
                    _stdError.Flush();

                    
                    byte[] bytes = _stdOutput.GetBuffer ();
                    string response = System.Text.ASCIIEncoding.ASCII.GetString(bytes, 0, (int) _stdOutput.Length);
                    if (!String.IsNullOrEmpty(response))
                        OnResponse(response, ResponseStyle.Output);

                    bytes = _stdError.GetBuffer();
                    response = System.Text.ASCIIEncoding.ASCII.GetString(bytes, 0, (int)_stdError.Length);
                    if (!String.IsNullOrEmpty(response))
                        OnResponse(response, ResponseStyle.Error);
                }
                catch (Exception e)
                {
                    // There should be no unhandled exceptions in the interactive session
                    // We catch all exceptions here, and just display it,
                    // and keep on going
                    OnResponse(_engine.FormatException(e), ResponseStyle.Error);
                }

                _currentStatement.Reset();

                /// write prompt
                OnPrompt(_engine.Sys.ps1.ToString());

                /// reset current command
                Reset();
            }
            else
            {
                // write a prompt for incomplete statements: Keep on reading input
                OnPrompt(_engine.Sys.ps2.ToString());

                /// reset current command
                Reset();

                // if AutoIndent is enabled, than add the indentation to the curent command
                if (_options.AutoIndent)
                {
                    int indentationLength = _currentStatement.AutoIndent;
                    string indentation = new string(' ', indentationLength);
                    _currentCommand.Append(indentation);
                    _currentCommandIndex = indentationLength;
                    OnTextChanged(indentation, indentationLength);
                }
            }


        }

        public void InputEntered(string input)
        {
            StringReader reader = new StringReader(input);
            string line = reader.ReadLine();
            while (line != null)
            {
                foreach (char c in line)
                    CharKeyPressed(c);

                EnterPressed();
                line = reader.ReadLine();
            }
        }

        private static bool TreatAsBlankLine(string line /*, int autoIndentSize */)
        {
            return String.IsNullOrEmpty(line) || line.Trim ().Length == 0;
            /*
            if (line.Length == 0) return true;
            if (autoIndentSize != 0 && line.Trim().Length == 0 && line.Length == autoIndentSize)
            {
                return true;
            }

            return false;
             */
        }

        public void Start()
        {
            OnPrompt (_engine.Sys.ps1.ToString());
        }

        public void UpPressed()
        {
            if (!_historyIterator.IsValid)
                _historyIterator.Reset();
            else if (!_historyIterator.HasPrevious)
                return;
            else
                _historyIterator.MovePrevious();

            _currentCommand.Length =0;
            _currentCommand .Append (_historyIterator.Current);
            _currentCommandIndex = _currentCommand.Length;

            OnTextChanged(_currentCommand.ToString(), _currentCommandIndex);
        }
        public void DownPressed()
        {
            if (!_historyIterator.IsValid)
                _historyIterator.Reset();
            else if (!_historyIterator.HasNext)
                return;
            else
                _historyIterator.MoveNext();

            _currentCommand.Length = 0;
            _currentCommand.Append (_historyIterator.Current);
            _currentCommandIndex = _currentCommand.Length;

            OnTextChanged(_currentCommand.ToString(), _currentCommandIndex);
        }
    }
}
