using System;
using System.Collections.Generic;
using System.Text;

namespace IronPython.Console
{
    [NUnit.Framework.TestFixture]
    public class DriverTest
    {
        interface IConsole
        {
            void driver_CursorMoved(Driver sender, Driver.CursorMovedEventArgs args);
            void driver_TextChanged(Driver sender, Driver.TextChangedEventArgs args);
            void driver_Response(Driver sender, Driver.ResponseEventArgs args);
            void driver_Prompt(Driver sender, Driver.ResponseEventArgs args);
        }

        [NUnit.Framework.Test]
        public void OneLinerTest()
        {
            Driver driver = new Driver();
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingPosition, 0);
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingText, "");
            
            driver.CharKeyPressed('x');
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingPosition, 1);
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingText, "x");

            driver.CharKeyPressed('=');
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingPosition, 2);
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingText, "x=");

            driver.CharKeyPressed('1');
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingPosition, 3);
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingText, "x=1");
            
            driver.EnterPressed();
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingPosition, 0);
            NUnit.Framework.Assert.AreEqual(driver.CurrentEditingText, "");
        }

        [NUnit.Framework.Test]
        public void IfStatementTest()
        {
            Driver driver = new Driver();
            
            NUnit.Mocks.DynamicMock mockConsole = new NUnit.Mocks.DynamicMock(typeof(IConsole));
            driver.Response += new Driver.ResponseEvent(((IConsole)mockConsole.MockInstance).driver_Response);
            driver.Prompt += new Driver.ResponseEvent(((IConsole)mockConsole.MockInstance).driver_Prompt);

            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs(">>> ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs("... ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs("... ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Response", driver, new Driver.ResponseEventArgs("hello world"+Environment.NewLine, Driver.ResponseStyle.Output));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs(">>> ", Driver.ResponseStyle.Prompt));

            driver.Start();
            driver.InputEntered("if (True) :");
            driver.InputEntered("print 'hello world'");
            driver.EnterPressed();

            mockConsole.Verify();
        }

        [NUnit.Framework.Test]
        public void IfElseStatementTest()
        {
            Driver driver = new Driver();

            NUnit.Mocks.DynamicMock mockConsole = new NUnit.Mocks.DynamicMock(typeof(IConsole));
            driver.Response += new Driver.ResponseEvent(((IConsole)mockConsole.MockInstance).driver_Response);
            driver.Prompt += new Driver.ResponseEvent(((IConsole)mockConsole.MockInstance).driver_Prompt);

            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs(">>> ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs("... ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs("... ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs("... ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs("... ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Response", driver, new Driver.ResponseEventArgs("hello world" + Environment.NewLine, Driver.ResponseStyle.Output));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs(">>> ", Driver.ResponseStyle.Prompt));

            driver.Start();
            driver.InputEntered("if (True) :");
            driver.InputEntered("print 'hello world'");
            driver.BackspacePressed(); /// for the auto-indent
            driver.InputEntered("else: ");
            driver.InputEntered("print 'goodbye cruel world'");
            driver.EnterPressed();

            mockConsole.Verify();
        }

        [NUnit.Framework.Test]
        public void AssignmentTest()
        {
            Driver driver = new Driver();

            NUnit.Mocks.DynamicMock mockConsole = new NUnit.Mocks.DynamicMock(typeof(IConsole));
            driver.Response += new Driver.ResponseEvent(((IConsole)mockConsole.MockInstance).driver_Response);
            driver.Prompt += new Driver.ResponseEvent(((IConsole)mockConsole.MockInstance).driver_Prompt);

            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs(">>> ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs(">>> ", Driver.ResponseStyle.Prompt));
            mockConsole.Expect("driver_Response", driver, new Driver.ResponseEventArgs("1" + Environment.NewLine, Driver.ResponseStyle.Output));
            mockConsole.Expect("driver_Prompt", driver, new Driver.ResponseEventArgs(">>> ", Driver.ResponseStyle.Prompt));

            driver.Start();
            driver.InputEntered("A = 1");
            driver.InputEntered("A");

            mockConsole.Verify();
        }

        [NUnit.Framework.Test]
        public void HistoryTest()
        {
            Driver driver = new Driver();

            NUnit.Mocks.DynamicMock mockConsole = new NUnit.Mocks.DynamicMock(typeof(IConsole));

            driver.Start();
            driver.InputEntered("A = 1");
            driver.InputEntered("B = 2");
            driver.InputEntered("C = 3");

            mockConsole.Expect("driver_TextChanged", driver, new Driver.TextChangedEventArgs("C = 3", "C = 3".Length));
            mockConsole.Expect("driver_TextChanged", driver, new Driver.TextChangedEventArgs("B = 2", "B = 2".Length));
            mockConsole.Expect("driver_TextChanged", driver, new Driver.TextChangedEventArgs("A = 1", "A = 1".Length));

            mockConsole.Expect("driver_TextChanged", driver, new Driver.TextChangedEventArgs("", 0));

            mockConsole.Expect("driver_TextChanged", driver, new Driver.TextChangedEventArgs("B = 2", "B = 2".Length));
            mockConsole.Expect("driver_TextChanged", driver, new Driver.TextChangedEventArgs("C = 3", "C = 3".Length));
            mockConsole.Expect("driver_TextChanged", driver, new Driver.TextChangedEventArgs("A = 1", "A = 1".Length));
           
            driver.TextChanged += new Driver.TextChangedEvent(((IConsole)mockConsole.MockInstance).driver_TextChanged);
            driver.UpPressed();
            driver.UpPressed();
            driver.UpPressed();
            driver.UpPressed();

            driver.EnterPressed();

            driver.DownPressed();
            driver.DownPressed();
            driver.DownPressed();
            driver.DownPressed();

            mockConsole.Verify();
        }

        [NUnit.Framework.Test]
        public void HistoryBasicUnitTest()
        {
            CommandHistory history = new CommandHistory();
            history.Record ("A = 1");
            history.Record ("B = 2");
            history.Record ("C = 3");

            CommandHistory.Iterator iterator = history.GetEnumerator();
            NUnit.Framework.Assert.AreEqual(iterator.Current, "C = 3");

            NUnit.Framework.Assert.IsFalse(iterator.HasNext);
            NUnit.Framework.Assert.IsTrue(iterator.HasPrevious);
            NUnit.Framework.Assert.IsTrue(iterator.MovePrevious ());

            NUnit.Framework.Assert.AreEqual(iterator.Current, "B = 2");

            NUnit.Framework.Assert.IsTrue(iterator.HasNext);
            NUnit.Framework.Assert.IsTrue(iterator.HasPrevious);
            NUnit.Framework.Assert.IsTrue(iterator.MovePrevious ());

            NUnit.Framework.Assert.AreEqual(iterator.Current, "A = 1");

            NUnit.Framework.Assert.IsTrue(iterator.HasNext);
            NUnit.Framework.Assert.IsFalse(iterator.HasPrevious);
            NUnit.Framework.Assert.IsTrue(iterator.MoveNext ());

            NUnit.Framework.Assert.AreEqual(iterator.Current, "B = 2");

            history.Record("D = 4");

            NUnit.Framework.Assert.IsTrue(iterator.HasNext);
            NUnit.Framework.Assert.IsTrue(iterator.HasPrevious);
            NUnit.Framework.Assert.IsTrue(iterator.MoveNext ());

            NUnit.Framework.Assert.AreEqual(iterator.Current, "C = 3");

            NUnit.Framework.Assert.IsTrue(iterator.HasNext);
            NUnit.Framework.Assert.IsTrue(iterator.HasPrevious);
            NUnit.Framework.Assert.IsTrue(iterator.MoveNext());

            NUnit.Framework.Assert.AreEqual(iterator.Current, "D = 4");

            NUnit.Framework.Assert.IsFalse(iterator.HasNext);
            NUnit.Framework.Assert.IsTrue(iterator.HasPrevious);

            history.Record("E = 5");
            iterator.Reset();

            NUnit.Framework.Assert.AreEqual(iterator.Current, "E = 5");
        }

        [NUnit.Framework.Test]
        public void HistoryCacheUnitTest()
        {
            CommandHistory history = new CommandHistory(3);
            history.Record("A = 1");
            history.Record("B = 2");
            history.Record("C = 3");
            history.Record("D = 4");

            CommandHistory.Iterator iterator = history.GetEnumerator();
            NUnit.Framework.Assert.AreEqual(iterator.Current, "D = 4");
            
            NUnit.Framework.Assert.IsTrue(iterator.MovePrevious ());
            NUnit.Framework.Assert.AreEqual(iterator.Current, "C = 3");

            NUnit.Framework.Assert.IsTrue(iterator.MovePrevious());
            NUnit.Framework.Assert.AreEqual(iterator.Current, "B = 2");

            NUnit.Framework.Assert.IsFalse(iterator.HasPrevious);
        }
    }
}
